# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Andrew Starr-Bochicchio <a.starr.b@gmail.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from locale import gettext as _

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk  # pylint: disable=E0611
gi.require_version('WebKit2', '4.0')
from gi.repository import WebKit2

import itertools
import re
import glob
import logging
logger = logging.getLogger('typecatcher')

from typecatcher_lib import Window
from typecatcher.AboutTypeCatcherDialog import AboutTypeCatcherDialog
from typecatcher.FindFonts import FindFonts
from typecatcher_lib.xdg import fontDir
from typecatcher_lib.helpers import running_gnome_shell
from typecatcher.DownloadFont import DownloadFont, UninstallFont, DownloadError
from typecatcher.html_preview import html_font_view, select_text_preview


# See typecatcher_lib.Window.py for more details about how this class works
class TypeCatcherWindow(Window):
    __gtype_name__ = "TypeCatcherWindow"

    def finish_initializing(self, builder):  # pylint: disable=E1002
        self.fonts = FindFonts()
        """Set up the main window"""
        super(TypeCatcherWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutTypeCatcherDialog

        # Code for other initialization actions should be added here.
        self.toolbar = builder.get_object("toolbar")
        context = self.toolbar.get_style_context()
        context.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
        self.view = WebKit2.WebView()
        webview = builder.get_object("webview")
        webview.add(self.view)
        htmlfile = html_font_view()
        self.view.load_html(htmlfile, "file:///")
        webview.show_all()

        webview_settings = self.view.get_settings()
        webview_settings.set_property(
            'enable-write-console-messages-to-stdout',
            True)
        self.view.connect("context-menu", self.disable_context_features)

        # the data in the model
        listmodel = builder.get_object("liststore")
        # append the values in the model
        for i in range(len(self.fonts)):
            listmodel.append(self.fonts[i])

        # a treeview to see the data stored in the model
        self.listview = builder.get_object("font_list")
        cell = Gtk.CellRendererText()
        col = Gtk.TreeViewColumn(_("Fonts"), cell, text=0)
        self.listview.append_column(col)
        self.listview.columns_autosize()
        # when a row is selected, it emits a signal
        self.listview.get_selection().connect("changed", self.on_select_changed)

        self.search_field = self.builder.get_object('search_field')
        completion = self.builder.get_object('entrycompletion')
        completion.set_model(listmodel)
        completion.set_text_column(0)
        self.search_field.set_completion(completion)
        completion.connect('match-selected', self.on_match_selected)

        self.mnu_save = self.builder.get_object('mnu_save')
        self.mnu_save.connect("activate", self.on_download_btn_clicked)

        self.text_menu = self.builder.get_object('text_menu')

        # Fallback to known icon for backwards compat
        icon_theme = Gtk.IconTheme.get_default()
        if icon_theme.has_icon('view-more-symbolic'):
            menu_icon = 'view-more-symbolic'
        else:
            logger.debug("Falling back to go-down-symbolic")
            menu_icon = 'go-down-symbolic'

        self.selector_icon = Gtk.Image.new()
        self.selector_icon.set_from_icon_name(menu_icon,
                                              Gtk.IconSize.LARGE_TOOLBAR)

        self.text_selector = Gtk.MenuButton.new()
        self.text_selector.set_popup(self.text_menu)
        self.text_selector.add(self.selector_icon)

        self.selector_tool_item = Gtk.ToolItem.new()
        self.selector_tool_item.add(self.text_selector)
        self.selector_tool_item.set_margin_left(4)

        self.text_selector.show()
        self.selector_tool_item.show()
        self.selector_icon.show()
        self.toolbar.insert(self.selector_tool_item, 6)

        self.text_selector.set_tooltip_text(_("Select text..."))

        radios = ['r1', 'r2', 'r3', 'r4', 'r5', 'r6', 'r7', 'r8']
        for position, item in enumerate(radios):
            item = self.builder.get_object(item)
            item.connect("toggled",
                         self.on_menu_choices_changed,
                         str(position + 1))
        self.text_content = "random"

        self.scale = self.builder.get_object('spinbutton')
        self.scale.set_value(36)
        self.scale.connect("value-changed", self.spin_moved)

        self.menubar = self.builder.get_object('menubar')
        # Hide menubar under GNOME
        if running_gnome_shell():
            self.menubar.hide()

    def spin_moved(self, widget):
        size = int(self.scale.get_value())
        text_size = "document.getElementById('text_preview').style.fontSize = '%s';" % size
        self.view.run_javascript(text_size)

    def on_menu_choices_changed(self, button, name):
        if name == "1":
            self.text_content = "random"
        elif name == "2":
            self.text_content = "ipsum"
        elif name == "3":
            self.text_content = "kafka"
        elif name == "4":
            self.text_content = "hgg"
        elif name == "5":
            self.text_content = "ggm"
        elif name == "6":
            self.text_content = "ralph"
        elif name == "7":
            self.text_content = "jj"
        elif name == "8":
            self.text_content = "custom"
        self.set_text()

    def set_text(self):
        t = select_text_preview(self.text_content)
        js_text = "document.getElementById('text_preview').innerHTML = '%s';" % t
        self.view.run_javascript(js_text)

    def js_exec(self):
        js_code = ["document.getElementById('start_page').style.display = 'None';",
                   """
                   document.getElementById('text_preview').style.fontFamily = "\'%s\'";
                   """  % (self.font)]
        font_loader = """WebFont.load({
            google: { families: [ '%s' ] },
            fontinactive: function(font, fvd) {
                console.log("Couldn't load " + font + " FVD:" + fvd);
                document.getElementById('start_page').style.display = 'None';
                document.getElementById('text_preview').style.display = 'None';
                document.getElementById('no_connect').style.display = 'block';
            },
        });""" % (self.font)
        show_text = [font_loader,
                     "document.getElementById('no_connect').style.display = 'None';",
                     "document.getElementById('text_preview').style.display = 'block';"]
        js_code.extend(show_text)
        if self.text_content == "random":
            self.set_text()
        for js in js_code:
            self.view.run_javascript(js)
        self.js_installed_check()

    def js_installed_check(self):
        if glob.glob(fontDir + self.font + '_*.*'):
            js_show = "document.getElementById('installed').style.display = 'block';"
            self.view.run_javascript(js_show)
        else:
            js_hide = "document.getElementById('installed').style.display = 'None';"
            self.view.run_javascript(js_hide)

    def download_failed(self):
        err = [
                "document.getElementById('start_page').style.display = 'None';",
                "document.getElementById('text_preview').style.display = 'None';",
                "document.getElementById('no_connect').style.display = 'block';"
        ]
        for js in err:
            self.view.run_javascript(js)

    def on_search_field_activate(self, widget):
        fonts = list(itertools.chain(*self.fonts))
        entered_text = self.search_field.get_text()
        matcher = re.compile(entered_text, re.IGNORECASE)
        if any(filter(matcher.match, fonts)):
            for position, item in enumerate(fonts):
                if item.lower() == entered_text.lower():
                    self.listview.set_cursor(position)
        else:
            pass

    def on_match_selected(self, completion, treemodel, treeiter):
        path = treemodel.get_path(treeiter)
        self.listview.set_cursor(path)

    def on_select_changed(self, selection):
        (model, iter) = selection.get_selected()
        self.font = model[iter][0]
        self.js_exec()

    def on_search_field_icon_press(self, widget, icon_pos, event):
        if icon_pos == Gtk.EntryIconPosition.SECONDARY:
            self.search_field.set_text("")

    def on_download_btn_clicked(self, button):
        try:
            try:
                DownloadFont(self.font, uri=None)
                self.js_installed_check()
            except DownloadError:
                self.download_failed()
        except AttributeError:
            head = _("This the install button.")
            message = _("Select a font on the left and press this button. The font will be installed for off-line use.")
            Window.info_dialog(self, head, message)

    def on_uninstall_btn_clicked(self, button):
        try:
            UninstallFont(self.font)
            self.js_installed_check()
        except AttributeError:
            head = _("This the uninstall button.")
            message = _("Select a font on the left and press this button. It will be removed from your system.")
            Window.info_dialog(self, head, message)

    def on_info_btn_clicked(self, button):
        try:
            info_url = "http://www.google.com/fonts/specimen/" + \
                self.font.replace(' ', '+', -1)
            Gtk.show_uri(None, info_url, 0)
        except AttributeError:
            head = _("This the info button.")
            message = _("Select a font on the left and press this button. A browser will open with further information about the chosen font.")
            Window.info_dialog(self, head, message)

    def on_mnu_save_as_activate(self, button):
        try:
            dialog = Gtk.FileChooserDialog("Please choose a file", self,
                                           Gtk.FileChooserAction.SAVE,
                                           (Gtk.STOCK_CANCEL,
                                            Gtk.ResponseType.CANCEL,
                                            Gtk.STOCK_SAVE,
                                            Gtk.ResponseType.OK)
                                           )
            filename = self.font + ".ttf"
            dialog.set_current_name(filename)
            response = dialog.run()
            if response == Gtk.ResponseType.OK:
                uri = dialog.get_filename()
                DownloadFont(self.font, uri)
            elif response == Gtk.ResponseType.CANCEL:
                pass
            dialog.destroy()
        except AttributeError:
            pass

    def on_text_selector_clicked(self, widget):
        self.text_menu.popup(None, None, None, None, 0,
                             Gtk.get_current_event_time())

    def disable_context_features(self, view, menu, event, hit_test_result):
        """Remove browser navigation from context menu.

           Itterates through the context menu and removes browser navigation
           items based on the enum values used to denote stock actions.
           Rather than disabling the menu, this allows us to retain the
           copy/paste items.

           See: https://lazka.github.io/pgi-docs/WebKit2-4.0/enums.html#WebKit2.ContextMenuAction
        """
        for item in menu.get_items():
            if item.get_stock_action() in [10, 11, 12, 13]:
                menu.remove(item)

        return False



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2014 Andrew Starr-Bochicchio <a.starr.b@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk, Gio # pylint: disable=E0611

from typecatcher import TypeCatcherWindow
from . helpers import get_builder, add_simple_action, running_gnome_shell

class TypeCatcherApplication(Gtk.Application):

    def __init__(self):
        Gtk.Application.__init__(self, application_id="apps.andrewsomething.typecatcher",
                                 flags=Gio.ApplicationFlags.FLAGS_NONE)
        self.connect("activate", self.on_activate)

    def on_activate(self, data=None):
        self.window = TypeCatcherWindow.TypeCatcherWindow()
        # Use ApplicationMenu under GNOME
        if running_gnome_shell():
            self.create_app_menu()
        self.add_window(self.window)
        self.window.show()

    def create_app_menu(self):
        builder = get_builder('AppMenu')
        appmenu = builder.get_object('appmenu')
        self.set_app_menu(appmenu)
        add_simple_action(self, 'save_as', 
            self.on_save_as_activated)
        add_simple_action(self, 'help', 
            self.on_help_activated)
        add_simple_action(self, 'about', 
            self.on_about_activated)
        add_simple_action(self, 'quit', 
            self.on_quit_activated)

    def on_save_as_activated(self, action, data=None):
        self.window.on_mnu_save_as_activate(action)

    def on_help_activated(self, action, data=None):
        self.window.on_mnu_contents_activate(action)

    def on_about_activated(self, action, data=None):
        self.window.on_mnu_about_activate(action)

    def on_quit_activated(self, action, data=None):
        self.window.on_mnu_close_activate(action)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Andrew Starr-Bochicchio <a.starr.b@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import json
import urllib.request, urllib.error, urllib.parse
import os
from typecatcher_lib.xdg import cacheDir

# Should the really be publicly viewable?
API_KEY = 'AIzaSyDpvpba_5RvJSvmXEJS7gZDezDaMlVTo4c'

def FindFonts():
        data = get_fonts_json()
        fonts = process_json(data)
        cache_json(data)
        return fonts


def get_fonts_json():
    try:
        req = urllib.request.Request(
          "https://www.googleapis.com/webfonts/v1/webfonts?key=%s" % API_KEY)
        opener = urllib.request.build_opener()
        data = opener.open(req).read().decode('utf8')
    except urllib.error.URLError:
        data = open_local_json()
    return data


def open_local_json():
    local_json = os.path.join(cacheDir + "webfonts.json")
    data = open(local_json).read()
    return data


def process_json(data):
    json_data = json.loads(data)
    fonts = []
    for n in json_data['items']:
        f = []
        f.append(str(n['family']))
        fonts.append(f)
    return fonts


def cache_json(data):
    local_json = os.path.join(cacheDir + "webfonts.json")
    if os.path.exists(cacheDir):
        with open(local_json, 'wb') as local_json:
            local_json.write(bytes(data, 'UTF-8'))
    else:
        os.makedirs(cacheDir)
        with open(local_json, 'wb') as local_json:
            local_json.write(bytes(data, 'UTF-8'))


def get_font_variants(family):
    data = open_local_json()
    json_data = json.loads(str(data))
    variants = []
    for n in json_data['items']:
        if n['family'] == family:
            for v in n['variants']:
                variants.append(str(v))
    return variants



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Andrew Starr-Bochicchio <a.starr.b@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

### DO NOT EDIT THIS FILE ###

from gi.repository import Gtk # pylint: disable=E0611
import logging
import subprocess
logger = logging.getLogger('typecatcher_lib')

from . helpers import get_builder, show_uri, get_help_uri

# This class is meant to be subclassed by TypeCatcherWindow.  It provides
# common functions and some boilerplate.
class Window(Gtk.ApplicationWindow):
    __gtype_name__ = "Window"

    # To construct a new instance of this method, the following notable 
    # methods are called in this order:
    # __new__(cls)
    # __init__(self)
    # finish_initializing(self, builder)
    # __init__(self)
    #
    # For this reason, it's recommended you leave __init__ empty and put
    # your initialization code in finish_initializing
    
    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated BaseTypeCatcherWindow object.
        """
        builder = get_builder('TypeCatcherWindow')
        new_object = builder.get_object("typecatcher_window")
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called while initializing this instance in __new__

        finish_initializing should be called after parsing the UI definition
        and creating a TypeCatcherWindow object with it in order to finish
        initializing the start of the new TypeCatcherWindow instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self, True)
        self.PreferencesDialog = None # class
        self.preferences_dialog = None # instance
        self.AboutDialog = None # class

    def on_mnu_contents_activate(self, widget, data=None):
        show_uri(self, "help:%s" % get_help_uri())

    def on_mnu_about_activate(self, widget, data=None):
        """Display the about box for typecatcher."""
        if self.AboutDialog is not None:
            about = self.AboutDialog() # pylint: disable=E1102
            response = about.run()
            about.destroy()

    def on_mnu_close_activate(self, widget, data=None):
        """Signal handler for closing the TypeCatcherWindow."""
        self.destroy()

    def on_destroy(self, widget, data=None):
        """Called when the TypeCatcherWindow is closed."""
        # Clean up code for saving application state should be added here.
        pass

    def on_mnu_report_activate(self, button):
        try: # Use apport if we can.
            subprocess.Popen(['ubuntu-bug', 'typecatcher'])
        except OSError: # If not, just open a browser.
            bug_url = "https://bugs.launchpad.net/typecatcher/+filebug"
            Gtk.show_uri(None, bug_url, 0)

    def on_mnu_trans_activate(self, button):
        bug_url = "https://translations.launchpad.net/typecatcher"
        Gtk.show_uri(None, bug_url, 0)

    def info_dialog(self, head, message):
        dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.INFO,
                                   Gtk.ButtonsType.OK,
                                   head)
        dialog.format_secondary_text(message)
        dialog.set_modal(True)
        dialog.run()
        dialog.destroy()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Andrew Starr-Bochicchio <a.starr.b@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os
from gi.repository import GLib

homeDir = os.environ.get('HOME')
confDir =  os.path.join(GLib.get_user_config_dir(), 'typecatcher/')
cacheDir =  os.path.join(GLib.get_user_cache_dir(), 'typecatcher/')
fontDir = os.path.join(homeDir, '.fonts/typecatcher/')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Andrew Starr-Bochicchio <a.starr.b@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import optparse

from locale import gettext as _

from typecatcher_lib import Application
from typecatcher_lib import set_up_logging, get_version

def parse_options():
    """Support for command line options"""
    parser = optparse.OptionParser(version="%%prog %s" % get_version())
    parser.add_option(
        "-v", "--verbose", action="count", dest="verbose",
        help=_("Show debug messages (-vv debugs typecatcher_lib also)"))
    (options, args) = parser.parse_args()

    set_up_logging(options)

def main():
    'constructor for your class instances'
    parse_options()

    # Run the application.    
    app = Application.TypeCatcherApplication()
    app.run(None)



# Apport integration for typecatcher
#
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# This file is in the public domain
### END LICENSE
import apport

def add_info(report):
    """add report info"""

    if not apport.packaging.is_distro_package(report['Package'].split()[0]):
        report['ThirdParty'] = 'True'
        report['CrashDB'] = 'typecatcher'



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Andrew Starr-Bochicchio <a.starr.b@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from locale import gettext as _
from gi.repository import Gtk
from typecatcher_lib.helpers import get_media_file
from random import choice


def html_font_view(font=None, text=None):

    start_page_icon = get_media_file("typecatcher.svg")

    try:
        con_icon_name = "nm-no-connection"
        con_theme = Gtk.IconTheme.get_default()
        con_info = con_theme.lookup_icon(con_icon_name, 64, 0)
        con_icon_uri = "file://" + con_info.get_filename()
    except AttributeError:
        con_icon_uri = get_media_file("no-connection.svg")

    try:
        installed_icon_name = "gtk-apply"
        installed_theme = Gtk.IconTheme.get_default()
        installed_info = installed_theme.lookup_icon(installed_icon_name, 64, 0)
        installed_icon_uri = "file://" + installed_info.get_filename()
    except AttributeError:
        installed_icon_uri = get_media_file("installed.svg")

    loader = get_media_file("ajax-loader.gif")

    text_preview = select_text_preview(text)

    html = """
<html>
  <head>

    <script src="https://ajax.googleapis.com/ajax/libs/webfont/1.6.26/webfont.js"></script>

    <style>
      body { font-size: 36px; }
      #installed { float: right; font-size: 12px; width:50px; text-align:center; display: None; }
      textarea { font: inherit; font-size: inherit; border: None; overflow: hidden; outline: none; width: 90%%; height: 100%%; }
      #text_preview { display: None; }
      #no_connect { text-align: center; display: None; font-size: 18px; }
      #start_page { text-align: center; bottom: 0px;}
     .wf-loading { height: 100%%; overflow: hidden; background: url(%s) center center no-repeat fixed;}
     .wf-loading * { opacity: 0; }

.wf-active body, .wf-inactive body {
    -webkit-animation: fade .25s ease;
            animation: fade .25s ease;
    -webkit-animation-fill-mode: both;
            animation-fill-mode: both;
}

@-webkit-keyframes fade {
    0%% { display: none; opacity: 0; }
    1%% { display: block; }
    100%%{ opacity: 1; }
}

@keyframes fade {
    0%% { display: none; opacity: 0; }
    1%% { display: block; }
    100%%{ opacity: 1; }
}
    </style>

  </head>

  <body>

     <div id="installed">
       <img src="%s" width=64 height=64>
       <p>%s</p>
     </div>

     <div id='no_connect'>
       <img src="%s" width=64 height=64 > <h1>%s</h1>
       <p>%s</p>
     </div>

    <div id='text_preview'>
      %s
    </div>

    <div id='start_page'>
      <img src="%s" width=128 height=128>
      <p>TypeCatcher</p>
    </div>

  </body>

</html>
""" % (loader, installed_icon_uri, _("Installed"),
       con_icon_uri, _("Font not available."),
       _("Please check your network connection."), text_preview,
       start_page_icon)

    return html


def select_text_preview(text):
    ipsum = """Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat."""
    kafka = _("One morning, when Gregor Samsa woke from troubled dreams, he found himself transformed in his bed into a horrible vermin. He lay on his armour-like back, and if he lifted his head a little he could see his brown belly, slightly domed and divided by arches into stiff sections.")
    hgg = _("Far out in the uncharted backwaters of the unfashionable end of the Western Spiral arm of the Galaxy lies a small unregarded yellow sun. Orbiting this at a distance of roughly ninety-eight million miles is an utterly insignificant little blue-green planet...")
    ggm = _("Many years later, as he faced the firing squad, Colonel Aureliano Buendia was to remember that distant afternoon when his father took him to discover ice.")
    ralph = _("I am an invisible man. No, I am not a spook like those who haunted Edgar Allan Poe; nor am I one of your Hollywood-movie ectoplasms. I am a man of substance, of flesh and bone, fiber and liquids — and I might even be said to possess a mind. I am invisible, understand, simply because people refuse to see me.")
    jj = _("Stately, plump Buck Mulligan came from the stairhead, bearing a bowl of lather on which a mirror and a razor lay crossed. A yellow dressinggown, ungirdled, was sustained gently behind him on the mild morning air.")

    text_pool = [ipsum, kafka, ggm, hgg, ralph, jj]

    if text is None or text == "random":
        selected_text = choice(text_pool)
        return "<p> %s </p>" % selected_text
    elif text == "ipsum":
        return "<p> %s </p>" % ipsum
    elif text == "kafka":
        return "<p> %s </p>" % kafka
    elif text == "hgg":
        return "<p> %s </p>" % hgg
    elif text == "ggm":
        return "<p> %s </p>" % ggm
    elif text == "ralph":
        return "<p> %s </p>" % ralph
    elif text == "jj":
        return "<p> %s </p>" % jj
    elif text == "custom":
        return "<textarea> %s </textarea>" % (_('Enter text...'))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Andrew Starr-Bochicchio <a.starr.b@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os
import urllib.request, urllib.error, urllib.parse
import glob
import json

from typecatcher_lib.xdg import fontDir
from typecatcher.FindFonts import open_local_json

WEBFONTS_API_URL = "http://fonts.googleapis.com/css?family="

class DownloadError(Exception):
    pass

def DownloadFont(font_name, uri):
    font_dict = extract_url(font_name)
    if font_dict is not None:
        for n in list(font_dict.items()):
            font_url = n[-1]
            variant = n[0]
            if font_url is not None:
                if uri is None:
                    font_dir = fontDir
                    write_font_file(font_url, font_dir, font_name, variant)
                else:
                    font_dir = os.path.dirname(uri)
                    write_font_file(font_url, font_dir, font_name, variant)
    else:
        raise DownloadError("The font could not be downloaded.")


def write_font_file(font_url, font_dir, font_name, variant):
    req = urllib.request.Request(font_url)
    r = urllib.request.urlopen(req)
    full_name = font_name + "_" + variant
    ext = os.path.splitext(font_url)[1]
    f = os.path.join(font_dir, full_name + ext)
    if not os.path.exists(font_dir):
        os.makedirs(font_dir)
    with open(f, 'wb') as f:
        f.write(r.read())


def extract_url(family):
    data = open_local_json()
    json_data = json.loads(str(data))
    for n in json_data['items']:
        if n['family'] == family:
            font_dict = dict(n['files'])
    return font_dict


def UninstallFont(font_name, font_dir=fontDir):
    font_file = glob.glob(font_dir + font_name + "_*.*")
    for f in font_file:
        os.remove(f)



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Andrew Starr-Bochicchio <a.starr.b@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

###################### DO NOT TOUCH THIS (HEAD TO THE SECOND PART) ######################

import os
import sys

try:
    import DistUtilsExtra.auto
except ImportError:
    print('To build typecatcher you need https://launchpad.net/python-distutils-extra', file=sys.stderr)
    sys.exit(1)
assert DistUtilsExtra.auto.__version__ >= '2.18', 'needs DistUtilsExtra.auto >= 2.18'

def update_config(libdir, values = {}):

    filename = os.path.join(libdir, 'typecatcher_lib/typecatcherconfig.py')
    oldvalues = {}
    try:
        fin = open(filename, 'r', encoding='utf-8')
        fout = open(filename + '.new', 'w', encoding='utf-8')

        for line in fin:
            fields = line.split(' = ') # Separate variable from value
            if fields[0] in values:
                oldvalues[fields[0]] = fields[1].strip()
                line = "%s = %s\n" % (fields[0], values[fields[0]])
            fout.write(line)

        fout.flush()
        fout.close()
        fin.close()
        os.rename(fout.name, fin.name)
    except (OSError, IOError) as e:
        print(("ERROR: Can't find %s" % filename))
        sys.exit(1)
    return oldvalues


def move_desktop_file(root, target_data, prefix):
    # The desktop file is rightly installed into install_data.  But it should
    # always really be installed into prefix, because while we can install
    # normal data files anywhere we want, the desktop file needs to exist in
    # the main system to be found.  Only actually useful for /opt installs.

    old_desktop_path = os.path.normpath(root + target_data +
                                        '/share/applications')
    old_desktop_file = old_desktop_path + '/typecatcher.desktop'
    desktop_path = os.path.normpath(root + prefix + '/share/applications')
    desktop_file = desktop_path + '/typecatcher.desktop'

    if not os.path.exists(old_desktop_file):
        print(("ERROR: Can't find", old_desktop_file))
        sys.exit(1)
    elif target_data != prefix + '/':
        # This is an /opt install, so rename desktop file to use extras-
        desktop_file = desktop_path + '/extras-typecatcher.desktop'
        try:
            os.makedirs(desktop_path)
            os.rename(old_desktop_file, desktop_file)
            os.rmdir(old_desktop_path)
        except OSError as e:
            print(("ERROR: Can't rename", old_desktop_file, ":", e))
            sys.exit(1)

    return desktop_file

def update_desktop_file(filename, target_pkgdata, target_scripts):

    try:
        fin = open(filename, 'r', encoding='utf-8')
        fout = open(filename + '.new', 'w', encoding='utf-8')

        for line in fin:
            if 'Exec=' in line:
                cmd = line.split("=")[1].split(None, 1)
                line = "Exec=%s" % (target_scripts + 'typecatcher')
                if len(cmd) > 1:
                    line += " %s" % cmd[1].strip()  # Add script arguments back
                line += "\n"
            fout.write(line)
        fout.flush()
        fout.close()
        fin.close()
        os.rename(fout.name, fin.name)
    except (OSError, IOError) as e:
        print(("ERROR: Can't find %s" % filename))
        sys.exit(1)

def compile_schemas(root, target_data):
    if target_data == '/usr/':
        return  # /usr paths don't need this, they will be handled by dpkg
    schemadir = os.path.normpath(root + target_data + 'share/glib-2.0/schemas')
    if (os.path.isdir(schemadir) and
            os.path.isfile('/usr/bin/glib-compile-schemas')):
        os.system('/usr/bin/glib-compile-schemas "%s"' % schemadir)


class InstallAndUpdateDataDirectory(DistUtilsExtra.auto.install_auto):
    def run(self):
        DistUtilsExtra.auto.install_auto.run(self)

        target_data = '/' + os.path.relpath(self.install_data, self.root) + '/'
        target_pkgdata = target_data + 'share/typecatcher/'
        target_scripts = '/' + os.path.relpath(self.install_scripts, self.root) + '/'

        values = {'__typecatcher_data_directory__': "'%s'" % (target_pkgdata),
                  '__version__': "'%s'" % self.distribution.get_version()}
        update_config(self.install_lib, values)

        desktop_file = move_desktop_file(self.root, target_data, self.prefix)
        update_desktop_file(desktop_file, target_pkgdata, target_scripts)
        compile_schemas(self.root, target_data)

        
##################################################################################
###################### YOU SHOULD MODIFY ONLY WHAT IS BELOW ######################
##################################################################################

DistUtilsExtra.auto.setup(
    name='typecatcher',
    version='0.3',
    license='GPL-3',
    author='Andrew Starr-Bochicchio',
    author_email='a.starr.b@gmail.com',
    description='Download Google webfonts for off-line use',
    long_description='TypeCatcher allows you to search, browse, and download Google webfonts for off-line use. You can preview fonts with adjustable size and text.',
    url='https://launchpad.net/typecatcher',
    cmdclass={'install': InstallAndUpdateDataDirectory}
    )




#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Andrew Starr-Bochicchio <a.starr.b@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import sys
import os.path
import unittest
sys.path.insert(0, os.path.realpath(os.path.join(os.path.dirname(__file__), "..")))
import tempfile
import json
import glob
from typecatcher import AboutTypeCatcherDialog
from typecatcher.DownloadFont import extract_url, write_font_file, UninstallFont
from typecatcher.FindFonts import process_json, cache_json, get_fonts_json, get_font_variants
from typecatcher_lib.xdg import cacheDir

class TestCases(unittest.TestCase):
    def setUp(self):
        self.AboutTypeCatcherDialog_members = [
        'AboutDialog', 'AboutTypeCatcherDialog', 'logger', 'logging']

        self.font_name = 'Abril Fatface'
        self.font_dict = {"regular": "http://themes.googleusercontent.com/static/fonts/abrilfatface/v5/X1g_KwGeBV3ajZIXQ9VnDojjx0o0jr6fNXxPgYh_a8Q.ttf"}
        self.font_list = [["Abel"], ["Abril Fatface"]]
        self.fake_font_dir = tempfile.mkdtemp() + "-fonts"

    def test_AboutTypeCatcherDialog_members(self):
        all_members = dir(AboutTypeCatcherDialog)
        public_members = [x for x in all_members if not x.startswith('_')]
        public_members.sort()
        self.assertEqual(self.AboutTypeCatcherDialog_members, public_members)

    def test_extract_url(self):
        returned_dict = extract_url(self.font_name)
        self.assertEqual(self.font_dict, returned_dict)

    def test_write_font_file(self):
        for n in list(self.font_dict.items()):
            font_url = n[-1]
            variant = n[0]
        write_font_file(font_url,  self.fake_font_dir,
                        self.font_name, variant)
        ext = os.path.splitext(font_url)[1]
        full_name = self.font_name + "_" + variant + ext
        downloaded_font = os.path.join( self.fake_font_dir, full_name)
        self.assertTrue(os.path.isfile(downloaded_font))

    def test_UninstallFont(self):
        font_file = self.fake_font_dir + self.font_name + "_normal-400.ttf"
        with open(font_file, 'wb') as f:
            f.write(bytes('Test', 'UTF-8'))
        UninstallFont(self.font_name, self.fake_font_dir)
        self.assertFalse(os.path.isfile(font_file))

    def test_process_json(self):
        returned_list = process_json(fake_json_data)
        self.assertEqual(self.font_list, returned_list)

    def test_cache_json(self):
        cache_json(fake_json_data)
        with open(os.path.join(cacheDir + "webfonts.json")) as f:
            self.assertEqual(f.read(), fake_json_data)

    def test_get_fonts_json(self):
        returned_data = get_fonts_json()
        self.assertTrue(json.loads(str(returned_data)))

    def test_get_font_variants(self):
        variants= get_font_variants(self.font_name)
        self.assertTrue(variants, ['regular'])

fake_json_data = """{
 "kind": "webfonts#webfontList",
 "items": [
  {
   "kind": "webfonts#webfont",
   "family": "Abel",
   "variants": [
    "regular"
   ],
   "subsets": [
    "latin"
   ],
   "version": "v3",
   "lastModified": "2012-07-25",
   "files": {
    "regular": "http://themes.googleusercontent.com/static/fonts/abel/v3/RpUKfqNxoyNe_ka23bzQ2A.ttf"
   }
  },
  {
   "kind": "webfonts#webfont",
   "family": "Abril Fatface",
   "variants": [
    "regular"
   ],
   "subsets": [
    "latin-ext",
    "latin"
   ],
   "version": "v5",
   "lastModified": "2012-07-25",
   "files": {
    "regular": "http://themes.googleusercontent.com/static/fonts/abrilfatface/v5/X1g_KwGeBV3ajZIXQ9VnDojjx0o0jr6fNXxPgYh_a8Q.ttf"
   }
  }
 ]
}"""

if __name__ == '__main__':
    unittest.main()
